# -*- coding: utf-8 -*-
import pytest

from django.http import HttpRequest as DjangoRequest
from webob import Request as WebobRequest
from werkzeug.wrappers import Request


def make_werkzeug_request(env, path):
    return Request(env)


def make_webob_request(env, path):
    return WebobRequest(dict(env))


def make_django_request(env, path):
    r = DjangoRequest()
    r.path = path
    r.META = dict(env)
    return r


@pytest.fixture(params=['werkzeug', 'webob', 'django'])
def make_req(request):
    return globals()['make_{0}_request'.format(request.param)]



# -*- coding: utf-8 -*-
import datetime as dt
import os
from decimal import Decimal
from os.path import join, dirname, realpath, abspath, normpath, isdir, isfile

from babel import dates, numbers, Locale
from markupsafe import Markup
from pytz import timezone, UTC

from allspeak import utils
from allspeak._compat import string_types
from allspeak.reader import get_data


LOCALES_DIR = 'locales'

DEFAULT_DATE_FORMATS = {
    'time': 'medium',
    'date': 'medium',
    'datetime': 'medium',
}

DEFAULT_LOCALE = 'en'


class I18n(object):

    def __init__(self, locales_dirs=None, get_request=None,
                 default_locale=DEFAULT_LOCALE, default_timezone=None,
                 date_formats=None, markup=Markup):
        """
        locales_dirs:
            list of paths that will be searched, in order,
            for the locales.

        get_request:
            a callable that returns the current request.

        default_locale:
            default locale (as a string or as a Babel.Locale instance).

        default_timezone:
            default timezone.

        date_formats:
            overwrite the defaults date formats.

        markup:
            overwrite the function used by `translate` to flags HTML code
            as 'safe'. `markupsafe.Markup` is used by default.

        """
        self.get_request = get_request

        if isinstance(locales_dirs, string_types):
            locales_dirs = [locales_dirs]
        locales_dirs = locales_dirs or [LOCALES_DIR]
        search_paths = []
        for p in locales_dirs:
            p = normpath(abspath(realpath(p)))
            if not isdir(p):
                p = dirname(p)
            search_paths.append(p)
        self.search_paths = search_paths
        self.set_defaults(default_locale, default_timezone)
        self.date_formats = DEFAULT_DATE_FORMATS.copy()
        if date_formats:
            self.date_formats.update(date_formats)
        self.markup = markup
        self.translations = {}

    def set_defaults(self, default_locale, default_timezone):
        """Set the default locale from the configuration as an instance of
        `Babel.Locale` and the default timezone as a `pytz.timezone` object.

        """
        default_locale = utils.normalize_locale(default_locale) or Locale(DEFAULT_LOCALE)
        self.default_locale = default_locale
        self.default_timezone = timezone(default_timezone or 'utc')

    def get_locale(self):
        """Returns the locale that should be used for this request as
        an instance of `Babel.Locale`.
        This returns the default locale if used outside of a request.

        """
        request = self.get_request and self.get_request()
        if not request:
            return self.default_locale
        return utils.get_request_locale(request, self.default_locale)

    def get_timezone(self):
        """Returns the timezone that should be used for this request as
        `pytz.timezone` object.  This returns the default timezone if used
        outside of a request or if no timezone was defined.

        """
        request = self.get_request and self.get_request()
        if not request:
            return self.default_timezone
        return utils.get_request_timezone(request, self.default_timezone)

    def load_language(self, path, locale):
        """From the given `path`, load the language file for the current or
        given locale.  If the locale has a territory attribute (eg: 'US') the
        the specific 'en-US' version will be tried first.

        """
        if locale.territory:
            filenames = [str(locale).replace('_', '-'), locale.language]
        else:
            filenames = [locale.language]
        filenames.append(str(self.default_locale).replace('_', '-'))

        for filename in filenames:
            cache_key = join(path, filename)
            cached = self.translations.get(cache_key)
            if cached:
                return cached
            filename = cache_key + '.yml'
            if isfile(filename):
                break
        else:
            return
        try:
            data = get_data(filename)
        except (IOError, AttributeError):
            return
        self.translations[cache_key] = data
        return data

    def find_keypath(self, key):
        """Based on the `key`, teturn the path of the language file and the
        subkey inside that file.

        """
        if ':' not in key:
            return self.search_paths[0], key

        path, subkey = key.split(':', 1)
        lpath = path.split('.')

        for root in self.search_paths:
            dirname, dirnames, filenames = next(os.walk(root))
            if lpath[0] in dirnames:
                break
        else:
            return None, None

        path = join(root, *lpath)
        if not isdir(path):
            return None, None
        return path, subkey

    def key_lookup(self, key, locale):
        """
        """
        path, subkey = self.find_keypath(key)
        if not (path and subkey):
            return None

        value = self.load_language(path, locale)
        if value is None:
            return None

        try:
            for k in subkey.split('.'):
                value = value.get(k)
                if value is None:
                    return None
            return value
        except (IndexError, ValueError):
            return None

    def translate(self, key, count=None, locale=None, **kwargs):
        """Load the translation for the given key using the current locale.

        If the value is a dictionary, and `count` is defined, uses the value
        whose key is that number.  If that key doesn't exist, a `'n'` key
        is tried instead.  If that doesn't exits either, an empty string is
        returned.

        The final value is formatted using `kwargs` (and also `count` if
        available) so the format placeholders must be named instead of
        positional.

        If the value isn't a dictionary or a string, is returned as is.

        Examples:

            >>> translate('hello_world')
            'hello %(what)s'
            >>> translate('hello_world', what='world')
            'hello world'
            >>> translate('a_list', what='world')
            ['a', 'b', 'c']

        """
        key = str(key)
        locale = utils.normalize_locale(locale) or self.get_locale()
        value = self.key_lookup(key, locale)
        if not value:
            return self.markup('<missing:%s>' % (key, ))

        if isinstance(value, dict):
            value = utils.pluralize(value, count)

        if isinstance(value, string_types):
            kwargs.setdefault('count', count)
            value = value % kwargs
            if key.endswith('_html'):
                return self.markup(value)

        return value

    @property
    def lazy_translate(self):

        class LazyWrapper(object):

            def __init__(self_, *args, **kwargs):
                self_.args = args
                self_.kwargs = kwargs

            def __repr__(self_):
                return self.translate(*self_.args, **self_.kwargs)

        return LazyWrapper

    def to_user_timezone(self, datetime, tzinfo=None):
        """Convert a datetime object to the user's timezone.  This
        automatically happens on all date formatting unless rebasing is
        disabled.  If you need to convert a `datetime.datetime` object at any
        time to the user's timezone (as returned by `get_timezone` this
        function can be used).

        """
        if datetime.tzinfo is None:
            datetime = datetime.replace(tzinfo=UTC)
        tzinfo = tzinfo or self.get_timezone()
        return tzinfo.normalize(datetime.astimezone(tzinfo))

    def to_utc(self, datetime, tzinfo=None):
        """Convert a datetime object to UTC and drop tzinfo.  This is the
        opposite operation to `to_user_timezone`.

        """
        if datetime.tzinfo is None:
            tzinfo = tzinfo or self.get_timezone()
            datetime = tzinfo.localize(datetime)
        return datetime.astimezone(UTC).replace(tzinfo=None)

    def _get_format(self, key, format):
        """A small helper for the datetime formatting functions.  Looks up
        format defaults for different kinds.

        """
        if format is None:
            format = self.date_formats.get(key)
        if format in ('short', 'medium', 'full', 'long'):
            rv = self.date_formats.get('%s.%s' % (key, format))
            if rv is not None:
                format = rv
        return format

    def _date_format(self, formatter, obj, format, rebase,
                     locale=None, tzinfo=None, **extra):
        """Internal helper that formats the date.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        extra = {}
        if formatter is not dates.format_date and rebase:
            extra['tzinfo'] = tzinfo or self.get_timezone()
        return formatter(obj, format, locale=locale, **extra)

    def format(self, value, *args, **kwargs):
        """Return a formatted `value` according to the detected type and
        given parameters.  It doesn't know anything about currency, percent or
        scientific formats, so use the other methods for those cases.

        """
        locale = kwargs.pop('locale', None)
        tzinfo = kwargs.pop('tzinfo', None)

        if isinstance(value, dt.date):
            if isinstance(value, dt.datetime):
                return self.format_datetime(value, locale=locale,
                                            tzinfo=tzinfo, *args, **kwargs)
            else:
                return self.format_date(value, locale=locale, tzinfo=tzinfo,
                                        *args, **kwargs)

        if isinstance(value, int):
            return self.format_number(value, locale=locale, *args, **kwargs)
        if isinstance(value, (float, Decimal)):
            return self.format_decimal(value, locale=locale, *args, **kwargs)

        if isinstance(value, dt.time):
            return self.format_time(value, locale=locale, tzinfo=tzinfo,
                                    *args, **kwargs)
        if isinstance(value, dt.timedelta):
            return self.format_timedelta(value, locale=locale, *args, **kwargs)

        return value

    def format_datetime(self, datetime=None, format=None, rebase=True,
                        locale=None, tzinfo=None):
        """Return a date formatted according to the given pattern.  If no
        `datetime.datetime` object is passed, the current time is
        assumed.  By default rebasing happens which causes the object to
        be converted to the users's timezone (as returned by
        `to_user_timezone`).  This function formats both date and
        time.

        The format parameter can either be `'short'`, `'medium'`,
        `'long'` or `'full'` (in which cause the language's default for
        that setting is used, or the default from the `Babel.date_formats`
        mapping is used) or a format string as documented by Babel.

        This function is also available in the template context as filter
        named `datetimeformat`.

        """
        format = self._get_format('datetime', format)
        return self._date_format(dates.format_datetime, datetime, format,
                                 rebase, locale=locale, tzinfo=tzinfo)

    def format_date(self, date=None, format=None, rebase=True,
                    locale=None, tzinfo=None):
        """Return a date formatted according to the given pattern.  If no
        `datetime.datetime` or `datetime.date` object is passed,
        the current time is assumed.  By default rebasing happens which causes
        the object to be converted to the users's timezone (as returned by
        `to_user_timezone`).  This function only formats the date part
        of a `datetime.datetime` object.

        The format parameter can either be `'short'`, `'medium'`,
        `'long'` or `'full'` (in which cause the language's default for
        that setting is used, or the default from the `Babel.date_formats`
        mapping is used) or a format string as documented by Babel.

        This function is also available in the template context as filter
        named `dateformat`.

        """
        if rebase and isinstance(date, dt.datetime):
            date = self.to_user_timezone(date, tzinfo=tzinfo)
        format = self._get_format('date', format)
        return self._date_format(dates.format_date, date, format, rebase,
                                 locale=locale, tzinfo=tzinfo)

    def format_time(self, time=None, format=None, rebase=True,
                    locale=None, tzinfo=None):
        """Return a time formatted according to the given pattern.  If no
        `datetime.datetime` object is passed, the current time is
        assumed.  By default rebasing happens which causes the object to
        be converted to the users's timezone (as returned by
        `to_user_timezone`).  This function formats both date and
        time.

        The format parameter can either be `'short'`, `'medium'`,
        `'long'` or `'full'` (in which cause the language's default for
        that setting is used, or the default from the Babel.date_formats`
        mapping is used) or a format string as documented by Babel.

        This function is also available in the template context as filter
        named `timeformat`.

        """
        format = self._get_format('time', format)
        return self._date_format(dates.format_time, time, format, rebase,
                                 locale=locale, tzinfo=tzinfo)

    def format_timedelta(self, datetime_or_timedelta, granularity='second',
                         locale=None):
        """Format the elapsed time from the given date to now or the given
        timedelta.

        This function is also available in the template context as filter
        named `timedeltaformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        if isinstance(datetime_or_timedelta, dt.datetime):
            datetime_or_timedelta = dt.datetime.utcnow(
            ) - datetime_or_timedelta
        return dates.format_timedelta(datetime_or_timedelta, granularity,
                                      locale=locale)

    def format_number(self, number, locale=None):
        """Return the given number formatted for the locale in the
        current request.

        number:
            the number to format

        return (unicode):
            the formatted number

        This function is also available in the template context as filter
        named `numberformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_number(number, locale=locale)

    def format_decimal(self, number, format=None, locale=None):
        """Return the given decimal number formatted for the locale in the
        current request.

        number:
            the number to format
        format:
            the format to use

        return (unicode):
            the formatted number

        This function is also available in the template context as filter
        named `decimalformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_decimal(number, format=format, locale=locale)

    def format_currency(self, number, currency, format=None, locale=None):
        """Return the given number formatted for the locale in the
        current request.

        number:
            the number to format
        currency:
            the currency code
        format:
            the format to use

        return (unicode):
            the formatted number

        This function is also available in the template context as filter
        named `currencyformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_currency(
            number, currency, format=format, locale=locale
        )

    def format_percent(self, number, format=None, locale=None):
        """Return a percent value formatted for the locale in the
        current request.

        number:
            the number to format
        format:
            the format to use

        return (unicode):
            the formatted percent number

        This function is also available in the template context as filter
        named `percentformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_percent(number, format=format, locale=locale)

    def format_scientific(self, number, format=None, locale=None):
        """Return value formatted in scientific notation for the locale in
        the current request.

        number:
            the number to format
        format:
            the format to use

        return (unicode):
            the formatted percent number

        This function is also available in the template context as filter
        named `scientificformat`.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_scientific(number, format=format, locale=locale)



# -*- coding: utf-8 -*-
from __future__ import print_function

from allspeak import utils
from babel import Locale
from werkzeug.test import EnvironBuilder

from .conftest import make_werkzeug_request, make_webob_request, make_django_request


def get_test_env(path, **kwargs):
    builder = EnvironBuilder(path=path, **kwargs)
    return builder.get_environ()


def get_test_request(make_req, path='/', **kwargs):
    env = get_test_env(path, **kwargs)
    return make_req(env, path)


def test_content_negotiation(make_req):
    available_locales = ['en', 'es', 'es-PE']

    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('es')

    headers = [('Accept-Language', 'fr; q=1.0, en-US; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('en')

    headers = [('Accept-Language', 'fr; q=1.0, es-PE; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('es', 'PE')

    headers = [('Accept-Language', 'fr; q=1.0, es-PE; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, ['ru']) is None

    headers = [('Accept-Language', 'martian; q=1.0, venus; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, ['klingon']) is None


def test_normalize_locale():
    from babel import Locale

    assert utils.normalize_locale(Locale('en', 'US')) == Locale('en', 'US')
    assert utils.normalize_locale('en-US') == Locale('en', 'US')
    assert utils.normalize_locale('en_US') == Locale('en', 'US')
    assert utils.normalize_locale('es') == Locale('es')
    assert utils.normalize_locale(('en', 'US')) == Locale('en', 'US')
    assert utils.normalize_locale(('es', )) == Locale('es')
    assert utils.normalize_locale('klingon') == None


def test_get_werkzeug_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_werkzeug_request, headers=headers)
    langs = utils.get_werkzeug_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_webob_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_webob_request, headers=headers)
    langs = utils.get_webob_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_django_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_django_request, headers=headers)
    langs = utils.get_django_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_pluralize():
    d = {
        0: u'No apples',
        1: u'One apple',
        3: u'Few apples',
        'n': u'%(count)s apples',
    }
    assert utils.pluralize(d, 0) == u'No apples'
    assert utils.pluralize(d, 1) == u'One apple'
    assert utils.pluralize(d, 3) == u'Few apples'
    assert utils.pluralize(d, 10) == u'%(count)s apples'

    d = {
        0: u'off',
        'n': u'on'
    }
    assert utils.pluralize(d, 3) == u'on'

    d = {
        0: u'off',
        'n': u'on'
    }
    assert utils.pluralize(d, 0) == u'off'
    assert utils.pluralize({}, 3) == u''



# -*- coding: utf-8 -*-
"""
Utilities for writing code that runs on Python 2 and 3.
"""
import sys


PY2 = sys.version_info[0] == 2


if PY2:
    string_types = (basestring, )
else:
    string_types = (str, )



# -*- coding: utf-8 -*-
import io
import os
import re
from setuptools import setup, find_packages


def get_path(*args):
    return os.path.join(os.path.dirname(__file__), *args)


def read_from(filepath):
    with io.open(filepath, 'rt', encoding='utf8') as f:
        return f.read()


def get_version(package):
    data = read_from(get_path(package, '__init__.py'))
    version = re.search(r"__version__\s*=\s*u?'([^']+)'", data).group(1)
    return str(version)


def get_description(package):
    data = read_from(get_path(package, '__init__.py'))
    desc = re.search('"""(.+)"""', data, re.DOTALL).group(1)
    return desc.strip()


def get_requirements(filename='requirements.txt'):
    data = read_from(get_path(filename))
    lines = map(lambda s: s.strip(), data.splitlines())
    return [l for l in lines if l and not l.startswith('#')]


setup(
    name='AllSpeak',
    version=get_version('allspeak'),
    author='Juan-Pablo Scaletti',
    author_email='juanpablo@lucumalabs.com',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    url='http://github.com/lucuma/allspeak',
    license='MIT license (http://www.opensource.org/licenses/mit-license.php)',
    description='Pythonic i18n/l10n system for WSGI-based web applications',
    long_description=get_description('allspeak'),
    install_requires=get_requirements(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding: utf-8 -*-
"""
========================
AllSpeak
========================

Pythonic (yet ironically inspired by Rails) i18n/l10n solution for humans
doing WSGI-based web applications.  It's flexible, easy to use and
—unlike gettext— independent of any external compilation tool.

Powered by the awesome Babel and pytz libraries for the l10n part.

The library is MIT Licensed and compatible with Werkzeug, Webob and Django
request objects, so you can use it with pretty much any WSGI-based framework.

See the documentation online at http://lucuma.github.com/allspeak

"""
import os
# Workaround for a OSX bug
if os.environ.get('LC_CTYPE', '').lower() == 'utf-8':
    os.environ['LC_CTYPE'] = 'en_US.utf-8'

from allspeak.i18n import I18n, LOCALES_DIR  # noqa
from allspeak.utils import negotiate_locale, get_preferred_locales  # noqa

__version__ = '0.5.7'



# -*- coding: utf-8 -*-
from babel import Locale, UnknownLocaleError
from pytz import timezone, UTC

from allspeak._compat import string_types


def normalize_locale(locale):
    if isinstance(locale, Locale):
        return locale
    try:
        if isinstance(locale, string_types):
            locale = locale.replace('_', '-')
            return Locale.parse(locale, sep='-')
        if isinstance(locale, tuple):
            return Locale(*locale)
    except UnknownLocaleError:
        return


def get_werkzeug_preferred_locales(request):
    """Return a list of preferred languages from a `werkzeug.wrappers.Request`
    instance.

    """
    languages = getattr(request, 'accept_languages', None)
    if languages:
        return list(languages.values())


def get_webob_preferred_locales(request):
    """Return a list of preferred languages from a `webob.Request` instance.

    """
    languages = getattr(request, 'accept_language', None)
    if languages:
        return list(languages)


def get_django_preferred_locales(request):
    """Take a `django.HttpRequest` instance and return a list of preferred
    languages from the headers.

    """
    meta = getattr(request, 'META', None)
    if not meta:
        return None
    header = request.META.get('HTTP_ACCEPT_LANGUAGE')
    if header:
        languages = [l.strip().split(';')[::-1] for l in header.split(',')]
        languages = sorted(languages)[::-1]
        return [l[1].strip() for l in languages]


def get_preferred_locales(request):
    return (get_werkzeug_preferred_locales(request) or
            get_webob_preferred_locales(request) or
            get_django_preferred_locales(request))


def negotiate_locale(request, available_locales):
    """From the available locales, negotiate the most adequate for the
    client, based on the "accept language" header.
    """
    preferred = get_preferred_locales(request)
    if preferred:
        available_locales = map(
            lambda l: l.replace('_', '-'),
            available_locales
        )
        # To ensure a consistent matching, Babel algorithm is used.
        return Locale.negotiate(preferred, available_locales, sep='-')


def get_request_timezone(request, default=UTC):
    """Returns the timezone that should be used for this request as a
    `DstTzInfo` instance.

    Tries the following in order:
    - an attribute called `'tzinfo'`
    - a GET argument called `'tzinfo'`
    - the provided default timezone

    """
    tzinfo = (
        getattr(request, 'tzinfo', None) or
        getattr(request, 'args', getattr(
                request, 'GET', {})).get('tzinfo')
    ) or default

    if isinstance(tzinfo, string_types):
        tzinfo = timezone(tzinfo)
    request.tzinfo = tzinfo
    return request.tzinfo


def get_request_locale(request, default):
    """Returns the locale that should be used for this request as a
    `babel.Locale` instance.

    Tries the following in order:
    - an request attribute called `'locale'`
    - a GET argument called `'locale'`
    - the default locale

    """
    locale = (
        getattr(request, 'locale', None) or
        getattr(request, 'args', getattr(request, 'GET', {})).get('locale')
    )
    request.locale = normalize_locale(locale) or default
    return request.locale


def pluralize(dic, count):
    """Takes a dictionary and a number and return the value whose key in
    the dictionary is that number.  If that key doesn't exist, a `'n'` key
    is tried instead.  If that doesn't exits either, an empty string is
    returned.  Examples:

        >>> dic = {
            0: u'No apples',
            1: u'One apple',
            3: u'Few apples',
            'n': u'%(count)s apples',
            }
        >>> pluralize(dic, 0)
        'No apples'
        >>> pluralize(dic, 1)
        'One apple'
        >>> pluralize(dic, 3)
        'Few apples'
        >>> pluralize(dic, 10)
        '%(count)s apples'
        >>> pluralize({0: 'off', 'n': 'on'}, 3)
        'on'
        >>> pluralize({0: 'off', 'n': 'on'}, 0)
        'off'
        >>> pluralize({}, 3)
        ''

    """
    if count is None:
        count = 0
    scount = str(count)
    return dic.get(count, dic.get(scount, dic.get('n', u'')))



# -*- coding: utf-8 -*-
from __future__ import print_function

import datetime as d
from decimal import Decimal
from os.path import join, dirname

from babel import Locale
from babel.dates import format_date, format_datetime, format_time
from babel.numbers import (
    format_currency, format_decimal, format_number,
    format_percent, format_scientific)
from markupsafe import Markup
from pytz import timezone, UTC
from werkzeug.test import EnvironBuilder

from allspeak import I18n


USE_WEBOB = False
USE_DJANGO = False

locales_dir = join(dirname(__file__), 'locales')


def get_test_env(path, **kwargs):
    builder = EnvironBuilder(path=path, **kwargs)
    return builder.get_environ()


def get_test_request(make_req, path='/', **kwargs):
    env = get_test_env(path, **kwargs)
    return make_req(env, path)


def test_search_paths():
    i18n = I18n(locales_dir)
    assert i18n.search_paths == [locales_dir]


def test_app_defaults():
    i18n = I18n(default_locale='es-PE', default_timezone='America/Lima')
    assert True
    assert Locale('es', 'PE') == i18n.get_locale()
    assert timezone('America/Lima') == i18n.get_timezone()


def test_overwrite_date_formats():
    i18n = I18n(date_formats={'date': 'fizzbuzz'})
    assert i18n.date_formats['date'] == 'fizzbuzz'


def test_request_settings(make_req):
    request = get_test_request(make_req)
    request.locale = 'en'
    request.tzinfo = 'US/Eastern'
    get_request = lambda: request

    i18n = I18n(get_request=get_request, default_locale='es-PE',
                default_timezone='America/Lima')

    assert Locale.parse('en') == i18n.get_locale()
    assert timezone('US/Eastern') == i18n.get_timezone()


def test_no_preffered_language(make_req):
    request = get_test_request(make_req, headers=[])
    get_request = lambda: request
    i18n = I18n(locales_dir, get_request, default_locale='es')
    assert Locale('es') == i18n.get_locale()


def test_load_language():
    i18n = I18n(default_locale='fr')

    data = i18n.load_language(locales_dir, Locale('es'))
    assert data['mytest']['greeting'] == u'Hola'
    data = i18n.load_language(locales_dir, Locale('es', 'PE'))
    assert data['mytest']['greeting'] == u'Habla'
    data = i18n.load_language(locales_dir, Locale('es', 'CO'))
    assert data['mytest']['greeting'] == u'Hola'
    data = i18n.load_language(locales_dir, Locale('pt'))
    assert data is None


def test_find_keypath():
    i18n = I18n(locales_dir)

    path, subkey = i18n.find_keypath('mytest.greeting')
    assert path == locales_dir
    assert subkey == 'mytest.greeting'

    path, subkey = i18n.find_keypath('sub:mytest.greeting')
    assert path == join(locales_dir, 'sub')
    assert subkey == 'mytest.greeting'


def test_key_lookup():
    i18n = I18n(locales_dir, default_locale='en')
    locale = Locale('es')

    assert i18n.key_lookup('mytest.greeting', locale) == u'Hola'
    assert i18n.key_lookup('mytest.bla', locale) is None
    assert i18n.key_lookup('sub:mytest.greeting', locale) == u'Hola mundo'


def test_translate():
    i18n = I18n(locales_dir, default_locale='es-PE')
    locale = Locale('es')

    assert i18n.translate('mytest.greeting', locale=locale) == u'Hola'
    assert i18n.translate('mytest.greeting') == u'Habla'
    assert i18n.translate('mytest.apple', 3, locale=locale) == u'Few apples'
    assert i18n.translate('mytest.apple', 10, locale=locale) == u'10 apples'
    assert i18n.translate('bla', locale=locale) == Markup(u'<missing:bla>')


def test_lazy_translate():
    i18n = I18n(locales_dir, default_locale='es-PE')
    locale = Locale('es')

    lazy = i18n.lazy_translate('mytest.greeting', locale=locale)
    assert lazy != u'Hola'
    assert repr(lazy) == u'Hola'


def test_to_user_timezone():
    i18n = I18n()
    tzinfo = timezone('US/Eastern')
    now = d.datetime.utcnow()
    result = i18n.to_user_timezone(now, tzinfo=tzinfo)
    expected = tzinfo.fromutc(now)
    assert result == expected


def test_to_utc():
    i18n = I18n()
    tzinfo = timezone('US/Eastern')
    now = d.datetime.utcnow()
    tznow = tzinfo.fromutc(now)
    assert i18n.to_utc(tznow) == now


def test_format():
    i18n = I18n()
    locale = 'en_US'
    tzinfo = UTC

    test_cases = [
        (456, format_number),
        (3.14159, format_decimal),
        (Decimal('3.14159'), format_decimal),
    ]
    for value, bf in test_cases:
        assert i18n.format(value, locale=locale) == bf(value, locale=locale)

    test_cases = [
        (d.datetime(2012, 7, 28, 3, 4, 5), format_datetime),
        (d.date.today(), format_date),
        (d.time(3, 4, 5), format_time),
    ]
    for value, bf in test_cases:
        result = i18n.format(value, locale=locale, tzinfo=tzinfo)
        expected = bf(value, locale=locale)
        assert result == expected


def test_more_formatters():
    i18n = I18n()
    locale = 'en_US'
    v = 231.456

    result = i18n.format_currency(v, 'USD', locale=locale)
    expected = format_currency(v, 'USD', locale=locale)
    assert result == expected

    result = i18n.format_percent(v, locale=locale)
    expected = format_percent(v, locale=locale)
    assert result == expected

    result = i18n.format_scientific(v, locale=locale)
    expected = format_scientific(v, locale=locale)
    assert result == expected
